@@ -15,6 +15,101 @@ DISCOVERY_TIMEOUT = 5
 DISCOVERY_MESSAGE = b"plzbridge?"
 BUFFER_SIZE = 2048
 HTTP_TIMEOUT = 15
+HUB_KEYS = [
+    "APLTS",
+    "MAC",
+    "GWDEVICE",
+    "GATEWAY",
+    "WMAC",
+    "plzbridge",
+    "WMODE",
+    "EGW",
+    "ECBL",
+    "WADR",
+    "CLOUD_ENABLED",
+    "EPR",
+    "LABEL",
+    "EMSK",
+    "WGW",
+    "ICONN",
+    "EBCST",
+    "WPR",
+    "WSSID",
+    "WENC",
+    "CBTYPE",
+    "EADR",
+    "WPWR",
+    "WMSK",
+    "EMAC",
+    "DNS",
+    "APLCONN",
+    "sendmsg",
+    "SYSTEM",
+    "WBCST",
+    "WCH",
+]
+
+PRODUCT_KEYS = [
+    "LSTATUS",
+    "F2LF",
+    "PUMP",
+    "PWR",
+    "CHRSTATUS",
+    "T5",
+    "T4",
+    "T3",
+    "T2",
+    "T1",
+    "F1V",
+    "FANLMINMAX",
+    "FDR",
+    "IN",
+    "OUT",
+    "F2V",
+    "MOD",
+    "DPT",
+    "APLWDAY",
+    "FWDATE",
+    "STATUS",
+    "F2L",
+    "SETP",
+    "DP",
+    "F1RPM",
+    "VER",
+    "MBTYPE",
+    "PSENSLTSH",
+    "STOVETYPE",
+    "NOMINALPWR",
+    "AUTONOMYTYPE",
+    "SPLMAX",
+    "CHRONOTYPE",
+    "PELLETTYPE",
+    "FAN2MODE",
+    "FAN2TYPE",
+    "SPLMIN",
+    "PSENSLMIN",
+    "PSENSLMAX",
+    "LABEL",
+    "PSENSTYPE",
+    "CONFIG",
+    "UICONFIG",
+    "MAINTPROBE",
+    "FLUID",
+    "SN",
+    "IGNERRORS",
+    "POWERTIME",
+    "SERVICETIME",
+    "OVERTMPERRORS",
+    "HEATTIME",
+    "ONTIME",
+    "IGN",
+    "DOORMOTION",
+    "MOT1COUNT",
+    "LIGHTCONT",
+    "DOORMOTOR",
+    "DOOR",
+    "LGHT",
+]
 
 # to be completed!!
 class PalComm(object):
@@ -23,7 +118,7 @@ class PalComm(object):
         server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
 
         # Enable broadcasting mode
-        #server.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
+        # server.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
 
         server.settimeout(DISCOVERY_TIMEOUT)
         server.sendto(message, (host, UDP_PORT))
@@ -198,6 +293,9 @@ class PalDiscovery(object):
         return True
 
 
+# this class represent the product, to get all the needed informations
+# to handle it data_config_json is the minimum required also
+# response_json is needed for more detailed info
 class Palazzetti(object):
     """Palazzetti HTTP class"""
 
@@ -210,16 +308,16 @@ class Palazzetti(object):
     data_config_json = None
     data_config_object = None
 
-    def __init__(self, host, title="uniqueID"):
+    def __init__(self, host, product_id="uniqueID"):
+        _LOGGER.debug("Init of class palazzetti")
+
         self.ip = host
         self.palsocket = PalComm()
         self.state = "online"
-        self.unique_id = title
+        self.unique_id = product_id
         self.data = {}
         self.data["title"] = self.unique_id
 
-        _LOGGER.debug("Init of class palazzetti")
-
         self.code_status = {
             0: "OFF",
             1: "OFF TIMER",
@@ -296,10 +394,6 @@ class Palazzetti(object):
             return False
 
         _LOGGER.debug("Executing command: {message}")
-        # response = False
-
-        # api_discovery=PalComm()
-        # _response = await api_discovery.async_getHTTP(self.ip, message)
         _response = await self.palsocket.async_getHTTP(self.ip, message)
 
         if not _response:
@@ -308,7 +402,7 @@ class Palazzetti(object):
             if self.response_json != None:
                 self.response_json.update({"icon": "mdi:link-off"})
             else:
-                self.response_json=json.loads('{"icon": "mdi:link-off"}')
+                self.response_json = json.loads('{"icon": "mdi:link-off"}')
             return False
 
         # merge the result with the exixting responnse_json
@@ -332,8 +426,8 @@ class Palazzetti(object):
             self.response_json_alls = _response
         elif message == "GET STDT":
             self.response_json_stdt = _response
-    
-    #only works with GET ALLS
+
+    # only works with GET ALLS
     async def __async_UDP_get_request(self, message):
         """ request the stove """
         _response_json = None
@@ -342,22 +436,22 @@ class Palazzetti(object):
         if message is None:
             return False
 
-        #_LOGGER.debug("Executing command: {message}")
+        # _LOGGER.debug("Executing command: {message}")
         _response = await self.palsocket.async_callUDP(self.ip, message)
-        #print(_response)
-        #one single retry
+        # print(_response)
+        # one single retry
         if _response is None:
-            #print("retry")
+            # print("retry")
             _response = await self.palsocket.async_callUDP(self.ip, message)
 
         if not _response:
-            #print("enter here")
+            # print("enter here")
             self.state = "offline"
             self.data["state"] = self.state
             if self.response_json != None:
                 self.response_json.update({"icon": "mdi:link-off"})
             else:
-                self.response_json=json.loads('{"icon": "mdi:link-off"}')
+                self.response_json = json.loads('{"icon": "mdi:link-off"}')
             return False
 
         # merge the result with the exixting responnse_json
@@ -375,14 +469,14 @@ class Palazzetti(object):
         self.__config_parse()
 
         if message == b"plzbridge?GET ALLS":
+            # print("Passa di qua: UDP GET_ALLS")
             self.data["status"] = self.code_status.get(
                 self.response_json["STATUS"], self.response_json["STATUS"]
             )
             self.response_json_alls = _response
 
-    # send request to stove for set commands
-    # why not async?
-    def __request_send(self, message):
+    # send sync request to stove for set commands
+    def __get_request(self, message):
         """ request the stove """
         _response_json = None
 
@@ -410,16 +504,16 @@ class Palazzetti(object):
                 # print("palazzetti.stove - com error")
                 self.state = "com error"
                 self.data["state"] = self.state
-                
+
                 if self.response_json != None:
                     self.response_json.update({"icon": "mdi:link-off"})
                 else:
-                    self.response_json=json.loads('{"icon": "mdi:link-off"}')
+                    self.response_json = json.loads('{"icon": "mdi:link-off"}')
 
                 _LOGGER.error(
                     "Error returned by CBox - retry in 2 seconds (" + message + ")"
                 )
-                
+
                 time.sleep(2)
                 retry = retry + 1
 
@@ -479,7 +573,7 @@ class Palazzetti(object):
         if _fan == None:
             raise InvalidFanError
 
-        self.async_get_alls()
+        # self.async_get_alls()
 
         _fan_limits = {}
 
@@ -496,7 +590,9 @@ class Palazzetti(object):
             _fan_limits["min"] = None
             _fan_limits["max"] = None
 
-        if (_fan_limits.get("min", None) == None) or (_fan_limits.get("max", None)):
+        if (_fan_limits.get("min", None) == None) or (
+            _fan_limits.get("max", None) == None
+        ):
             raise InvalidFanLimitsError
 
         if _fan < _fan_limits.get("min") or _fan > _fan_limits.get("max"):
@@ -530,7 +626,7 @@ class Palazzetti(object):
 
         _command = {"FAN_1": "SET RFAN", "FAN_2": "SET FN3L", "FAN_3": "SET FN4L"}
 
-        command = f'{_command.get("FAN_" + "" + fan, None)} {value}'
+        command = f'{_command.get("FAN_" + "" + str(fan), None)} {value}'
 
         if command is None:
             raise InvalidFanOutOfRange
@@ -542,28 +638,6 @@ class Palazzetti(object):
         asset_capabilities = asset_parser.parsed_data
         self.data_config_object = asset_capabilities
 
-    def get_setpoint(self):
-        """Get target temperature for climate"""
-        if self.response_json == None or self.response_json["SETP"] == None:
-            return
-
-        return self.response_json["SETP"]
-
-    # get generic KEY in the datas
-    # if key doesn't exist returns None
-    # TODO: Is it possible to remove this function?
-    # BYNOW does not generate risks
-    def get_key(self, mykey="STATUS"):
-        """Get target temperature for climate"""
-        if (
-            self.response_json == None
-            or (mykey in self.response_json) == False
-            or self.response_json[mykey] == None
-        ):
-            return
-
-        return self.response_json[mykey]
-
     # TODO: Is it possible to remove this function?
     # BYNOW it has been limited to setpoint only
     async def set_parameters(self, datas):
@@ -587,16 +661,6 @@ class Palazzetti(object):
         self.data["label"] = value
         self.response_json.update({"LABEL": value})
 
-    async def async_set_fan_silent_mode(self):
-
-        command = self.__build_fan_command(1, 0)
-
-        if self.data_config_object.flag_has_fan_zero_speed_fan == True:
-            command = "SET SLNT 1"
-
-        if await self.__async_get_request(command) == False:
-            raise SendCommandError
-
     async def async_set_power(self, value):
 
         self.__validate_power(value)
@@ -610,6 +674,26 @@ class Palazzetti(object):
         self.data["powr"] = value
         self.response_json.update({"POWR": value})
 
+    async def async_set_fan_silent_mode(self):
+
+        command = self.__build_fan_command(1, 0)
+
+        if self.data_config_object.flag_has_fan_zero_speed_fan == True:
+            command = "SET SLNT 1"
+
+        if await self.__async_get_request(command) == False:
+            raise SendCommandError
+
+    def set_fan_silent_mode(self):
+
+        command = self.__build_fan_command(1, 0)
+
+        if self.data_config_object.flag_has_fan_zero_speed_fan == True:
+            command = "SET SLNT 1"
+
+        if self.__get_request(command) == False:
+            raise SendCommandError
+
     async def async_set_fan_auto_mode(self, fan=1):
 
         value = "7"  # Auto Mode
@@ -618,6 +702,14 @@ class Palazzetti(object):
         if await self.__async_get_request(command) == False:
             raise SendCommandError
 
+    def set_fan_auto_mode(self, fan=1):
+
+        value = "7"  # Auto Mode
+        command = self.__build_fan_command(fan, value)
+
+        if self.__get_request(command) == False:
+            raise SendCommandError
+
     async def async_set_fan_high_mode(self, fan=1):
 
         value = "6"  # High Mode
@@ -626,6 +718,14 @@ class Palazzetti(object):
         if await self.__async_get_request(command) == False:
             raise SendCommandError
 
+    def set_fan_high_mode(self, fan=1):
+
+        value = "6"  # High Mode
+        command = self.__build_fan_command(fan, value)
+
+        if self.__get_request(command) == False:
+            raise SendCommandError
+
     async def async_set_fan(self, fan, value):
 
         self.__validate_fan(fan, value)
@@ -635,6 +735,15 @@ class Palazzetti(object):
         if await self.__async_get_request(command) == False:
             raise SendCommandError
 
+    def set_fan(self, fan, value):
+
+        self.__validate_fan(fan, value)
+
+        command = self.__build_fan_command(fan, value)
+
+        if self.__get_request(command) == False:
+            raise SendCommandError
+
     async def async_set_light(self, value):
         if (value == None) or (type(value) is not bool):
             raise InvalidLightError
@@ -644,6 +753,15 @@ class Palazzetti(object):
         if await self.__async_get_request(command) == False:
             raise SendCommandError
 
+    def set_light(self, value):
+        if (value == None) or (type(value) is not bool):
+            raise InvalidLightError
+
+        command = f"SET LGHT {str(1 if value == True else 0)}"
+
+        if self.__get_request(command) == False:
+            raise SendCommandError
+
     async def async_set_door(self, value):
         if (value == None) or (type(value) is not bool):
             raise InvalidDoorError
@@ -678,7 +796,7 @@ class Palazzetti(object):
 
     def power_on(self):
 
-        if self.__request_send("GET STAT") == False:
+        if self.__get_request("GET STAT") == False:
             raise SendCommandError
 
         if self.data_config_object.flag_error_status == True:
@@ -692,12 +810,12 @@ class Palazzetti(object):
 
         command = "CMD ON"
 
-        if self.__request_send(command) == False:
+        if self.__get_request(command) == False:
             raise SendCommandError
 
     def power_off(self):
 
-        if self.__request_send("GET STAT") == False:
+        if self.__get_request("GET STAT") == False:
             raise SendCommandError
 
         if self.data_config_object.flag_error_status == True:
@@ -711,25 +829,96 @@ class Palazzetti(object):
 
         command = "CMD OFF"
 
-        if self.__request_send(command) == False:
+        if self.__get_request(command) == False:
             raise SendCommandError
 
     # retuens list of states: title, state, ip
-    def get_data_states(self):
+    def get_data_states(self) -> list:
         return self.data
 
     # retuens JSON with all keys of GET ALLS, GET STDT and GET CNTR
-    def get_data_json(self):
+    def get_data_json(self) -> json:
         return self.response_json
 
+    # retuens JSON specific for hub with all keys of GET ALLS, GET STDT
+    def get_cb_data_json(self) -> json:
+        newList = {
+            k: self.response_json[k] for k in HUB_KEYS if k in self.response_json
+        }
+        newList.update({"IP": self.ip})
+        # return json.dumps(newList)
+        return newList
+
+    # retuens JSON specific for product with all keys of GET ALLS, GET STDT and GET CNTR
+    def get_prod_data_json(self) -> json:
+        newList = {
+            k: self.response_json[k] for k in PRODUCT_KEYS if k in self.response_json
+        }
+        newList.update(
+            {
+                "STATE": self.code_status.get(
+                    self.response_json["STATUS"], self.response_json["STATUS"]
+                )
+            }
+        )
+        # return json.dumps(newList)
+        return newList
+
     # returns JSON with configuration keys
-    def get_data_config_json(self):
+    def get_data_config_json(self) -> json:
         return vars(self.data_config_object)
 
+    # returns OBJECT with configuration keys
+    def get_data_config_object(self):
+        return self.data_config_object
+
     # returns JSON with the DATA key content of the last GET ALLS
-    def get_data_alls_json(self):
+    def get_data_alls_json(self) -> json:
         return self.response_json_alls
 
     # returns JSON with the DATA key content of the last GET STDT
-    def get_data_stdt_json(self):
+    def get_data_stdt_json(self) -> json:
         return self.response_json_stdt
+
+    # returns int with setpoint
+    def get_setpoint(self) -> int:
+        """Get target temperature for climate"""
+        if self.response_json == None or self.response_json["SETP"] == None:
+            return
+
+        return int(self.response_json["SETP"])
+
+    # get generic KEY in the datas
+    # if key doesn't exist returns None
+    # TODO: Is it possible to remove this function?
+    # BYNOW does not generate risks
+    def get_key(self, mykey="STATUS"):
+        """Get target temperature for climate"""
+        if (
+            self.response_json == None
+            or (mykey in self.response_json) == False
+            or self.response_json[mykey] == None
+        ):
+            return
+
+        return self.response_json[mykey]
+
+    @property
+    def product_id(self) -> str:
+        """Return unique ID of product"""
+        return self.unique_id
+
+    @property
+    def hub_id(self) -> str:
+        """Return unique ID of the connectivity hub: ConnBox or BioCC"""
+        return self.response_json["MAC"].replace(":", "_")
+
+    @property
+    def hub_isbiocc(self) -> bool:
+        """Return True if hub is BioCC else is ConnBox"""
+        return self.response_json["CBTYPE"] == "ET4W"
+
+    @property
+    def online(self) -> bool:
+        """Return True if product is online"""
+        return self.state == "online"
\ No newline at end of file
